use std::collections::HashSet;

use actix_cors::Cors;
use actix_web::{rt::task::spawn_blocking, App, HttpServer};
use botdiscord::config::{parse_local_config, Config};
use serenity::{
    async_trait,
    client::{Context, EventHandler},
    http::Http,
    model::{
        channel::Message,
        gateway::{GatewayIntents, Ready},
    },
    Client,
};

struct Handler;

#[async_trait]
impl EventHandler for Handler {
    async fn message(&self, ctx: Context, msg: Message) {
        if msg.author.bot {
            return;
        }
        if msg.content == "!ping" {
            // Sending a message can fail, due to a network error, an
            // authentication error, or lack of permissions to post in the
            // channel, so log to stdout when some error happens, with a
            // description of it.
            if let Err(why) = msg.channel_id.say(&ctx.http, "Pong!").await {
                println!("Error sending message: {:?}", why);
            }
            return;
        }
    }

    async fn ready(&self, _: Context, ready: Ready) {
        println!("{} is connected!", ready.user.name);
    }
}

fn start_bot(config: Config) {
    if config.token != "" {
        spawn_blocking(move || {
            let rt = tokio::runtime::Handle::current();
            rt.block_on(async {
                let local = tokio::task::LocalSet::new();
                let _ = local
                    .run_until(async move {
                        let intents = GatewayIntents::GUILD_MESSAGES
                            | GatewayIntents::DIRECT_MESSAGES
                            | GatewayIntents::MESSAGE_CONTENT
                            | GatewayIntents::GUILD_VOICE_STATES
                            | GatewayIntents::GUILDS
                            | GatewayIntents::GUILD_MEMBERS;
                        let http = Http::new(&config.token);
                        let (_owners, _bot_id) = match http.get_current_application_info().await {
                            Ok(info) => {
                                let mut owners = HashSet::new();
                                match info.owner {
                                    Some(user) => {
                                        owners.insert(user.id);
                                        println!("{}", user.global_name.unwrap())
                                    }
                                    None => {}
                                }

                                (owners, info.id)
                            }
                            Err(why) => panic!("Could not access application info: {:?}", why),
                        };
                        let mut client = Client::builder(&config.token, intents)
                            .event_handler(Handler)
                            .await
                            .expect("Err creating client");
                        if let Err(why) = client.start().await {
                            println!("Client error: {why:?}");
                        }
                    })
                    .await;
            })
        });
        println!("JOJO 8");
    }
}

#[actix_web::main] // or #[tokio::main]
async fn main() -> std::io::Result<()> {
    let config = parse_local_config();
    let port = config.port;
    start_bot(config.clone());
    HttpServer::new(|| {
        let cors = Cors::default()
            .allow_any_header()
            .allow_any_method()
            .allow_any_origin();
        App::new().wrap(cors)
    })
    .bind(("0.0.0.0", port))?
    .run()
    .await?;
    Ok(())
}
